################################################################################
# Created       : Martin Foster
# Modified      : 08-Nov-2005
################################################################################
#
# Rotate - Rotation and display of banners and HTML templates
# Copyright (C) 2000-2006  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

package Ethereal::Rotate;					# Package definition


################################################################################
# Sub-Routines
################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	#####################
	# Data members
	my $class    = shift;					# Self reference
	my $self;						# Self referencing hash


	#####################
	# Program area

	# Assign hash
	%{$self} = @_;


	# Ensure definition
	# Database
	die('Database library has not been passed!') unless (defined($self->{'DB'}));
	die('Zone must be passed!') unless (defined($self->{'ZONE'}));

	# General defintions
	$self->{'STYLE'} = (defined($self->{'STYLE'})) ? $self->{'STYLE'} : 'random';
	$self->{'DISPL'} = (defined($self->{'DISPL'})) ? $self->{'DISPL'} : 'return';
	
	
	# Bess to form class
	bless $self, $class;
	return $self;
}



#####################
# Pull
#
# Performs the actual task of requesting the templates, modifying them if need be then
# returning the templates for display.

sub Pull
{
	#####################
	# Data members
	my $self     = shift;					# Reference to self

	my $database = $self->{'DB'};				# Database handle
	my $display  = $self->{'DISPL'};			# Display method
	my $style    = $self->{'STYLE'};			# Randomization style
	my $zone     = $self->{'ZONE'};				# Zone of influence

	my $conduct;						# Conduct of parser
	my $data;						# Data sheet
	my $count;						# Number of entries
	my $name;						# Name of template

	my %tags;						# Generic tag holder

	#####################
	# Program area

	# Pull template
	# Time based rotation
	if ($style eq 'time')
	{
		# Evaluate statement
		eval
		{
			# Selection process
			($name,
			 $data, 
			 $conduct) = $database->DataGet("SELECT
			 	 RotateName,
				 RotateData,
				 RotateConduct
				FROM Rotate
				WHERE RotateZone=?
			  	  AND RotateTimestamp IS NOT NULL
				ORDER BY RotateTimestamp, RotateName LIMIT 1
				FOR UPDATE OF Rotate", $zone);
			
			# Update timestamp
			$database->Write("UPDATE Rotate
				SET RotateTimestamp=LOCALTIMESTAMP
				WHERE RotateName=?
				  AND RotateZone=?", $name, $zone);

			# Commit changes
			$database->Commit();
		};

		# Verification
		if ($@)
		{
			# Issue warning
			warn("Transaction aborted: $@");

			# Undo incomplete changes
			$database->Rollback();
		}
	}

	# Random style retrieval
	else
	{
		# Selection process
		($data, 
		 $conduct) = $database->DataGet("SELECT
			 RotateData,
			 RotateConduct
			FROM Rotate
			WHERE RotateZone=?
			  AND RotateTimestamp IS NOT NULL
			ORDER BY RANDOM() LIMIT 1", $zone);
	}
	
	# Safety catch
	return '' unless (defined($data));


	# Template handling
	if ($conduct ne 'static')
	{
		# Pre-define
		my ($gid, $wid, $wname, $wimage, $wlayer, $wrate, $wtype, $wsub, $sgenre, $wpenname);

		# Hash
		$database->GetHashGallery(\%tags);
		
		# Image galleries
		if ($conduct eq 'image')
 		{
			# Loop with failsafe
			for ($count=0; $count < 5; $count++)
			{
				# Pull template
				($gid,
				 $wid,
				 $wlayer,
				 $wname,
				 $wimage,
			    	 $wrate,
			    	 $wpenname,
			 	 $wtype,
			 	 $wsub) = $database->DataGet("SELECT
				    	 g.GalleryID,
					 w.WorkID,
					 w.WorkLayer,
				    	 w.WorkName,
					 w.WorkImageThumbnail,
					 g.GalleryRating,
					 g.GalleryPenName,
					 g.GalleryType,
					 g.GallerySub
					FROM Work w, Gallery g
					WHERE w.GalleryID = g.GalleryID
					  AND g.GalleryType = 'image'
					  AND g.GalleryRating != ?
					  AND g.GalleryPrivacy = 'no'
					  AND w.WorkImageThumbnail IS NOT NULL
					  AND g.GalleryPenName = (SELECT GalleryPenName 
					    FROM Gallery 
					    WHERE GalleryType='image' 
					    GROUP BY GalleryPenName 
					    ORDER BY RANDOM() LIMIT 1)
					ORDER BY RANDOM() LIMIT 1", $tags{'SetRestricted'}
				);

				# Bypass safety mech
				last if (defined($gid));
			}

			# Only if necessary
			if (defined($gid))
			{
				# Pull necessary genre
				($wgenre) = $database->DataGet("SELECT GenreName
					FROM GalleryGenre
					WHERE GalleryID=?
					  AND GenreType=?
					  AND GenreSub=?",

				 $gid,
				 $wtype,
				 $wsub
				);
				
				# Safety
				$wlayer = (defined($wlayer)) ? $wlayer : 0;

				
				# Create link
				my $url = $tags{'SetScript'}
				        . $tags{'LnkGallery'} . '/'
			                . $wtype  . '/'
				        . $wsub   . '/'
				        . $wgenre . '/'
				       	. $gid    . '/'
				       	. $wlayer . '/'
					. $wid;

				# Search and replace
				$data =~ s/MNAME/$wname/gs;
				$data =~ s/MPENN/$wpenname/gs;
				$data =~ s/MRATE/$wrate/gs;
				$data =~ s/LTHMB/$wimage/gs;
				$data =~ s/LWORK/$url/gs;
			}

			else
			{
				# Nothing to display
				$data = undef;
			}
		}

		# Text galleries
		elsif ($conduct eq 'text')
 		{
			# Loop with failsafe
			for ($count=0; $count < 5; $count++)
			{
				# Pull template
				($gid,
				 $wid,
				 $wlayer,
				 $wname,
				 $wtext,
		    		 $wrate,
		    		 $wpenname,
			 	 $wtype,
			 	 $wsub) = $database->DataGet("SELECT
				    	 g.GalleryID,
					 w.WorkID,
					 w.WorkLayer,
				    	 w.WorkName,
					 w.WorkText,
					 g.GalleryRating,
					 g.GalleryPenName,
					 g.GalleryType,
					 g.GallerySub
					FROM Work w, Gallery g
					WHERE w.GalleryID = g.GalleryID
					  AND g.GalleryType = 'text'
					  AND g.GalleryRating != ?
					  AND g.GalleryPrivacy = 'no'
					  AND w.WorkText IS NOT NULL
					  AND g.GalleryPenName = (SELECT GalleryPenName 
					    FROM Gallery 
					    WHERE GalleryType='text' 
					    GROUP BY GalleryPenName 
					    ORDER BY RANDOM() LIMIT 1)
					ORDER BY RANDOM() LIMIT 1", $tags{'SetRestricted'}
				);

				# Bypass safety mech
				last if (defined($gid));
			}

			# Only if necessary
			if (defined($gid))
			{
				# Pull necessary genre
				($wgenre) = $database->DataGet("SELECT GenreName
					FROM GalleryGenre
					WHERE GalleryID=?
					  AND GenreType=?
					  AND GenreSub=?",

				 $gid,
				 $wtype,
				 $wsub
				);
				
				# Safety
				$wlayer = (defined($wlayer)) ? $wlayer : 0;

				
				# Only grab except
				$wtext = substr(DBI::neat($wtext, 252), 1, -1);
			

				# Safety
				# Html and special
				$wtext =~ s/<[^>]*>//gs;
				$wtext =~ s/\&.*?\;//gs;

				# Comments
				$wtext =~ s/<!-|-->//gs;


				# Create link
				my $url = $tags{'SetScript'}
				        . $tags{'LnkGallery'} . '/'
			                . $wtype  . '/'
				        . $wsub   . '/'
				        . $wgenre . '/'
				       	. $gid    . '/'
				       	. $wlayer . '/'
					. $wid;

				# Search and replace
				$data =~ s/MNAME/$wname/gs;
				$data =~ s/MPENN/$wpenname/gs;
				$data =~ s/MRATE/$wrate/gs;
				$data =~ s/MTEXT/$wtext/gs;
				$data =~ s/LWORK/$url/gs;
			}

			else
			{
				# Nothing to display
				$data = undef;
			}
		}
	}

	# Safety
	$data = (defined($data)) ? $data : '';
	
	
	# Exits
	# Standard behaviour
	return $data if ($display eq 'return');

	# Print and return
	print $data;
	return 1;
}

1;
